/*
	Base Object declaration
	(c) Copyright 1992/1994 Marco Savegnago IW3FQG
*/

#ifndef __TBOBJECT_H
#define __TBOBJECT_H

#include 	"tdef.h"
#include	<stddef.h>

class TBobject
{

public:
	TBobject();

	virtual ~TBobject();

	static void destroy( TBobject* );
	virtual void shutDown();

	virtual BOOL isValid();
	virtual BOOL operator! ();

protected:

	BOOL valid;
};

inline TBobject::TBobject()
{
	valid = TRUE;
}

inline void TBobject::destroy( TBobject *o )
{
	if( o != 0 )
		o->shutDown();

	delete o;
}

inline TBobject::~TBobject()
{
}

inline void TBobject::shutDown()
{
}

inline BOOL TBobject::isValid()
{
	return valid;
}

inline BOOL TBobject::operator! ()
{
	return BOOL(valid != TRUE);
}

class TBNSCollection : public TBobject
{

public:

	TBNSCollection( ccIndex aLimit, ccIndex aDelta );
	~TBNSCollection();

	virtual void shutDown();

	void *at( ccIndex index );
    virtual ccIndex indexOf( void *item );

	void atFree( ccIndex index );
	void atRemove( ccIndex index );
	void remove( void *item );
	 void* remove(WORD itemNo = 0);

    void removeAll();
    void free( void *item );
	void freeAll();

	void atInsert( ccIndex index, void *item );
	void atPut( ccIndex index, void *item );
    virtual ccIndex insert( void *item );

    static void error( ccIndex code, ccIndex info );

    void *firstThat( ccTestFunc Test, void *arg );
	void *lastThat( ccTestFunc Test, void *arg );
	void forEach( ccAppFunc action, void *arg );

    void pack();
    virtual void setLimit( ccIndex aLimit );

    ccIndex getCount()
		{ return count; }

	BOOL&	getShouldDelete()
    {
    	return shouldDelete;
    }

	void setShouldDelete(BOOL should)
    {
		shouldDelete = should;
    }

	void *last()
	{
		return (count) ? items[count - 1] : NULL;
	}

	void *first()
	{
		return	(count) ? items[0] : NULL;
	}

	void *next(void *item)
	{
		int current = indexOf(item);
		return ((current + 1) < getCount()) ? items[current+1] : NULL;
	}

	void *prev(void *item)
	{
		int current = indexOf(item);
		return ((current - 1) >= 0) ? items[current-1] : NULL;
	}

protected:

	TBNSCollection();

	void **items;
	ccIndex count;
	ccIndex limit;
	ccIndex delta;
	BOOL shouldDelete;

private:

	virtual void freeItem( void *item );

};

class TBNSSortedCollection: public virtual TBNSCollection
{

public:

	TBNSSortedCollection( ccIndex aLimit, ccIndex aDelta) :
		TBNSCollection( aLimit, aDelta ), duplicates(FALSE)
			{ delta = aDelta; setLimit( aLimit ); }

	virtual BOOL search( void *key, ccIndex& index );

	virtual ccIndex indexOf( void *item );
	virtual ccIndex insert( void *item );

	BOOL duplicates;
	virtual void *keyOf( void *item );

protected:

private:

	virtual int compare( void *key1, void *key2 ) = 0;

};

#endif

